-- ElvUI Companions Datatext <Combined Datatext>
-- By Lockslap, US - Bleeding Hollow
local E, L, V, P, G	= unpack(ElvUI)
local DT			= E:GetModule("DataTexts")
local LibQTip		= LibStub("LibQTip-1.0")

local join		= string.join
local split		= string.split
local band		= bit.band
local wipe		= table.wipe
local random	= math.random

local Tip
local displayString = ""
local colorString = "|cff%s%s|r"

local specialMounts = {
	[30174]		= true,	-- riding turtle
	[64731]		= true,	-- sea turtle
	[26054]		= true,	-- aq'40 bugs
	[25953]		= true,	-- ditto
	[26056]		= true,	-- same
	[26055]		= true,	-- mhm
}

local function ToHex(color)
	return ("%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255)
end

local function GetMountColor(flags)
	local ground, flyer, swimmer = band(flags, 0x01) ~= 0, band(flags, 0x02) ~= 0, band(flags, 0x08) ~= 0
	
	if swimmer == true and ground ~= true and flyer ~= true then
		return mountdb.swimColor
	elseif ground == true and flyer ~= true then
		return mountdb.groundColor
	elseif flyer == true then
		return mountdb.flyingColor
	else
		return mountdb.specialColor
	end
end

local function IsModifierDown()
	if mountdb.modifier == "shift" then
		return IsShiftKeyDown() and true or false
	elseif mountdb.modifier == "ctrl" then
		return IsControlKeyDown() and true or false
	elseif mountdb.modifier == "alt" then
		return IsAltKeyDown() and true or false
	else
		return false
	end
end

local function CompanionType()
	if mountdb.default == "mounts" then
		return not IsModifierDown() and "MOUNT" or "CRITTER"
	else
		return not IsModifierDown() and "CRITTER" or "MOUNT"
	end
end

local function GetHeaderName()
	if mountdb.default == "mounts" then
		return not IsModifierDown() and L["Mounts"] or L["Pets"]
	else
		return not IsModifierDown() and L["Pets"] or L["Mounts"]
	end
end

local function CompanionClick(frame, info, button)
	local id, active = split(":", info)
	if tonumber(active) == 1 then
		DismissCompanion(CompanionType())
	else
		CallCompanion(CompanionType(), tonumber(id))
	end
	Tip:Hide()
end

local function OnEnter(self)
	if InCombatLockdown() then return end
	local pets = {}
	local header = GetHeaderName()
	local companion = CompanionType()
	
	if LibQTip:IsAcquired("ElvUI_Companions") then
		Tip:Clear()
	else
		Tip = LibQTip:Acquire("ElvUI_Companions", mountdb.columns)
		Tip:SmartAnchorTo(self)
		Tip:SetAutoHideDelay(0.1, self)
		Tip:SetScript("OnShow", function(self) self:SetTemplate("Transparent") end)
	end
	
	line = Tip:AddHeader()
	Tip:SetCell(line, 1, ("|cffffffff%s|r"):format(header), "LEFT")
	Tip:AddSeparator()
	
	if GetNumCompanions(companion) == 0 then
		line = Tip:AddLine()
		Tip:SetCell(line, 1, ("|cffff0000No %s Found!|r"):format(header), "CENTER")
	else
		wipe(pets)
		
		for i = 1, GetNumCompanions(companion) do
			local _, name, spell, _, active, flags = GetCompanionInfo(companion, i)
			local colored
			
			if companion == "CRITTER" then
				colored = colorString:format(ToHex(active == 1 and mountdb.selectColor or mountdb.petColor), name)
			else
				if active == 1 then
					colored = colorString:format(ToHex(mountdb.selectColor), name)
				elseif specialMounts[spell] ~= nil then
					colored = colorString:format(ToHex(mountdb.specialColor), name)
				else
					colored = colorString:format(ToHex(GetMountColor(flags)), name)
				end
			end
			
			pets[#pets + 1] = {
				id		= i,
				raw		= name,
				name	= colored,
				active	= active,
			}
		end
		
		-- no funny business
		if #pets == 0 then return end
		
		-- sort by the companions raw name, w/o any coloring
		sort(pets, function(a, b) return a["raw"] < b["raw"] end)
		
		local i = 1
		for index, value in pairs(pets) do
			if i == 1 then line = Tip:AddLine() end
			Tip:SetCell(line, i, value.name, "LEFT", nil, nil, nil, nil, nil, 200)
			Tip:SetCellScript(line, i, "OnMouseUp", CompanionClick, tostring(value.id) .. ":" .. tostring(value.active))
			
			if i == mountdb.columns then
				i = 1
			else
				i = i + 1
			end
		end
	end
	
	Tip:UpdateScrolling()
	Tip:Show()
end

local function OnEvent(self, event, ...)
	self.text:SetFormattedText(displayString,
		GetHeaderName(),
		GetNumCompanions(CompanionType()))
end

local function OnUpdate(self, elapsed)
	self.text:SetFormattedText(displayString,
		GetHeaderName(),
		GetNumCompanions(CompanionType()))
end

local menuFrame = CreateFrame("Frame", "CompanionsDatatextRightClickMenu", E.UIParent, "UIDropDownMenuTemplate")
local function RandomClick(self, arg1, arg2, checked)
	menuFrame:Hide()
	if arg1 == "pet" then
		CallCompanion("CRITTER", random(1, GetNumCompanions("CRITTER")))
	else
		local rand
		for i = 1, GetNumCompanions("MOUNT") do
			rand = random(1, GetNumCompanions("MOUNT"))
			local _, name, _, _, _, flags = GetCompanionInfo("MOUNT", rand)
			local ground, flyer = band(flags, 0x01) ~= 0, band(flags, 0x02) ~= 0
			if arg2 == "flying" and flyer == true then
				CallCompanion("MOUNT", rand)
				break
			elseif arg2 == "ground" and ground == true and flying ~= true then
				CallCompanion("MOUNT", rand)
				break
			end
		end
	end
end
local menuList = {
	{text = L["Random Companion"], isTitle = true, notCheckable = true},
	{text = L["Mounts"], hasArrow = true, notCheckable = true, menuList = {
			{text = L["Flyer"], notCheckable = true, arg1 = "mount", arg2 = "flying", func = RandomClick},
			{text = L["Ground"], notCheckable = true, arg1 = "mount", arg2 = "ground", func = RandomClick},
	}},
	{text = L["Pet"], notCheckable = true, arg1 = "pet", func = RandomClick},
}

local function Click(self, button)
	Tip:Hide()
	if button == "LeftButton" then
		ToggleSpellBook(IsShiftKeyDown() and "companions" or "mount")
	elseif button == "RightButton" then
		--menuList[2].
		EasyMenu(menuList, menuFrame, "cursor", 0, 0, "MENU", 2)
	end
end

local function ValueColorUpdate(hex, r, g, b)
	displayString = join("", "|cffffffff%s:|r", " ", hex, "%d|r")
end
E["valueColorUpdateFuncs"][ValueColorUpdate] = true

DT:RegisterDatatext("Companions", {"PLAYER_ENTERING_WORLD"}, OnEvent, OnUpdate, Click, OnEnter)