-- ElvUI Companions Datatext <Mounts Datatext>
-- By Lockslap, US - Bleeding Hollow
local E, L, V, P, G	= unpack(ElvUI)
local DT			= E:GetModule("DataTexts")
local LibQTip		= LibStub("LibQTip-1.0")

local join		= string.join
local split		= string.split
local band		= bit.band
local wipe		= table.wipe
local random	= math.random

local Tip
local displayString = ""
local colorString = "|cff%s%s|r"

local specialMounts = {
	[30174]		= true,	-- riding turtle
	[64731]		= true,	-- sea turtle
	[26054]		= true,	-- aq'40 bugs
	[25953]		= true,	-- ditto
	[26056]		= true,	-- same
	[26055]		= true,	-- mhm
}

local function ToHex(color)
	return ("%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255)
end

local function GetMountColor(flags)
	local ground, flyer, swimmer = band(flags, 0x01) ~= 0, band(flags, 0x02) ~= 0, band(flags, 0x08) ~= 0
	
	if swimmer == true and ground ~= true and flyer ~= true then
		return mountdb.swimColor
	elseif ground == true and flyer ~= true then
		return mountdb.groundColor
	elseif flyer == true then
		return mountdb.flyingColor
	else
		return mountdb.specialColor
	end
end

local function OnEvent(self, event, ...)
	self.text:SetFormattedText(displayString, GetNumCompanions("MOUNT"))
end

local function OnUpdate(self, elapsed)
	self.text:SetFormattedText(displayString, GetNumCompanions("MOUNT"))
end

local function MountClick(frame, info, button)
	local id, active = split(":", info)
	if tonumber(active) == 1 then
		DismissCompanion("MOUNT")
	else
		CallCompanion("MOUNT", tonumber(id))
	end
	Tip:Hide()
end

local menuFrame = CreateFrame("Frame", "MountsDatatextRightClickMenu", E.UIParent, "UIDropDownMenuTemplate")
local function RandomMount(self, arg1, arg2, checked)
	local rand
	menuFrame:Hide()
	
	for i = 1, GetNumCompanions("MOUNT") do
		rand = random(1, GetNumCompanions("MOUNT"))
		local _, _, _, _, _, flags = GetCompanionInfo("MOUNT", rand)
		local ground, flyer = band(flags, 0x01) ~= 0, band(flags, 0x02) ~= 0
		if arg1 == "flying" and flyer == true then
			CallCompanion("MOUNT", rand)
			break
		elseif arg == "ground" and ground == true and flying ~= true then
			CallCompanion("MOUNT", rand)
			break
		end
	end
end
local menuList = {
	{text = L["Summon Random Mount"], isTitle = true, notCheckable = true},
	{text = L["Flying Mount"], notCheckable = true, arg1 = "flying", func = RandomMount},
	{text = L["Ground Mount"], notCheckable = true, arg1 = "ground", func = RandomMount},
}

local function Click(self, button)
	Tip:Hide()
	if button == "RightButton" then
		EasyMenu(menuList, menuFrame, "cursor", 0, 0, "MENU", 2)
	else
		ToggleSpellBook("mount")
	end
end

local function OnEnter(self)
	if InCombatLockdown() then return end
	local mounts = {}
	
	if LibQTip:IsAcquired("ElvUI_Mounts") then
		Tip:Clear()
	else
		Tip = LibQTip:Acquire("ElvUI_Mounts", mountdb.columns)
		Tip:SmartAnchorTo(self)
		Tip:SetAutoHideDelay(0.1, self)
		Tip:SetScript("OnShow", function(self) self:SetTemplate("Transparent") end)
	end
	
	line = Tip:AddHeader()
	Tip:SetCell(line, 1, L["|cffffffffMounts|r"], "LEFT")
	Tip:AddSeparator()
	
	if GetNumCompanions("MOUNT") == 0 then
		line = Tip:AddLine()
		Tip:SetCell(line, 1, L["|cffff0000No Mounts Found!|r"], "CENTER")
	else
		wipe(mounts)
		
		for i = 1, GetNumCompanions("MOUNT") do
			local _, name, spell, _, active, flags = GetCompanionInfo("MOUNT", i)
			local colored
			
			if active == 1 then
				colored = colorString:format(ToHex(mountdb.selectColor), name)
			elseif specialMounts[spell] ~= nil then
				colored = colorString:format(ToHex(mountdb.specialColor), name)
			else
				colored = colorString:format(ToHex(GetMountColor(flags)), name)
			end
			
			mounts[#mounts + 1] = {
				id		= i,
				raw		= name,
				name	= colored,
				active	= active
			}
		end
		
		if #mounts == 0 then return end
		sort(mounts, function(a, b) return a["raw"] < b["raw"] end)
	
		local i = 1
		for index, value in pairs(mounts) do
			if i == 1 then line = Tip:AddLine() end
			Tip:SetCell(line, i, value.name, "LEFT", nil, nil, nil, nil, nil, 200)
			Tip:SetCellScript(line, i, "OnMouseUp", MountClick, tostring(value.id) .. ":" .. tostring(value.active))
			
			if i == mountdb.columns then
				i = 1
			else
				i = i + 1
			end
		end
	end
	
	Tip:UpdateScrolling()
	Tip:Show()
end

local function ValueColorUpdate(hex, r, g, b)
	displayString = join("", "|cffffffffMounts:|r", " ", hex, "%d|r")
end
E["valueColorUpdateFuncs"][ValueColorUpdate] = true

DT:RegisterDatatext("Mounts", {"PLAYER_ENTERING_WORLD"}, OnEvent, OnUpdate, Click, OnEnter)