-- ElvUI Companions Datatext <Combined Datatext>
-- By Lockslap, US - Bleeding Hollow
local E, L, V, P, G	= unpack(ElvUI)

local defaults = {
	["default"]			= "mounts",
	["modifier"]		= "shift",
	["columns"]			= 3,
	
	["swimColor"]		= {r = 0,		g = 0.44,	b = 0.87},		-- rare (blue)
	["groundColor"]		= {r = 0.64,	g = 0.21,	b = 0.93},		-- epic (purple)
	["flyingColor"]		= {r = 1.0,		g = 0.5,	b = 0},			-- legendary (orange)
	["specialColor"]	= {r = 0.0,		g = 1.0,	b = 0},			-- uncommon (green)
	["petColor"]		= {r = 0.01,	g = 0.64,	b = 0.8},		-- turqoise-ish
	["selectColor"]		= {r = 1.0,		g = 0.0,	b = 0.56},		-- pink-ish
}

-- hack to make sure the settings are always up to date
mountdb = mountdb or {}
for index, value in pairs(defaults) do
	if type(mountdb[index]) == "nil" then
		mountdb[index] = value
	end
end

E.Options.args.datatexts.args.companions = {
	type		= "group",
	order		= 25,
	name		= L["Companions"],
	guiInline	= true,
	get			= function(info) return mountdb[info[#info]] end,
	set			= function(info, value) mountdb[info[#info]] = value; DT:LoadDataTexts() end,
	args		= {
		modifier = {
			type	= "select",
			order	= 1,
			name	= L["Modifier"],
			desc	= L["Key to hold to swap between mounts and companions."],
			values	= {
				["alt"]		= L["Alt"],
				["ctrl"]	= L["Ctrl"],
				["shift"]	= L["Shift"],
			},
		},
		default = {
			type	= "select",
			order	= 2,
			name	= L["Combined Default"],
			desc	= L["The default companion (mounts or pets) to show without a modifier.  This only applies to the combined datatext."],
			values	= {
				["mounts"]	= L["Mounts"],
				["pets"]	= L["Non-Combat Pets"],
			},
		},
		columns = {
			type	= "range",
			order	= 3,
			name	= L["Columns"],
			desc	= L["The number of columns in the tooltip."],
			min = 1, max = 5, step = 1,
			set		= function(info, value)
				mountdb[info[#info]] = value
				for key, tooltip in LibQTip:IterateTooltips() do
					if key == "ElvUI_Companions" or key == "ElvUI_Mounts" or key == "ElvUI_Pets" then
						LibQTip:Release(tooltip)
					end
				end
			end,
		},
		swimColor		= {
			type	= "color",
			order	= 4,
			name	= L["Swimming Mount Color"],
			desc	= L["Color to use for swimming mounts."],
			get		= function(info) return mountdb[info[#info]].r, mountdb[info[#info]].g, mountdb[info[#info]].b end,
			set		= function(info, r, g, b)
				mountdb[info[#info]].r = r
				mountdb[info[#info]].g = g
				mountdb[info[#info]].b = b
			end,
		},
		groundColor		= {
			type	= "color",
			order	= 5,
			name	= L["Ground Mount Color"],
			desc	= L["Color to use for ground mounts."],
			get		= function(info) return mountdb[info[#info]].r, mountdb[info[#info]].g, mountdb[info[#info]].b end,
			set		= function(info, r, g, b)
				mountdb[info[#info]].r = r
				mountdb[info[#info]].g = g
				mountdb[info[#info]].b = b
			end,
		},
		flyingColor		= {
			type	= "color",
			order	= 6,
			name	= L["Flying Mount Color"],
			desc	= L["Color to use for flying mounts."],
			get		= function(info) return mountdb[info[#info]].r, mountdb[info[#info]].g, mountdb[info[#info]].b end,
			set		= function(info, r, g, b)
				mountdb[info[#info]].r = r
				mountdb[info[#info]].g = g
				mountdb[info[#info]].b = b
			end,
		},
		specialColor	= {
			type	= "color",
			order	= 7,
			name	= L["Special Mount Color"],
			desc	= L["Color to use for the \"special\" mounts.  Like the bugs from AQ."],
			get		= function(info) return mountdb[info[#info]].r, mountdb[info[#info]].g, mountdb[info[#info]].b end,
			set		= function(info, r, g, b)
				mountdb[info[#info]].r = r
				mountdb[info[#info]].g = g
				mountdb[info[#info]].b = b
			end,
		},
		petColor		= {
			type	= "color",
			order	= 8,
			name	= L["Pet Color"],
			desc	= L["Color to use for pets."],
			get		= function(info) return mountdb[info[#info]].r, mountdb[info[#info]].g, mountdb[info[#info]].b end,
			set		= function(info, r, g, b)
				mountdb[info[#info]].r = r
				mountdb[info[#info]].g = g
				mountdb[info[#info]].b = b
			end,
		},
		selectColor		= {
			type	= "color",
			order	= 9,
			name	= L["Selected Color"],
			desc	= L["Color for selected companions."],
			get		= function(info) return mountdb[info[#info]].r, mountdb[info[#info]].g, mountdb[info[#info]].b end,
			set		= function(info, r, g, b)
				mountdb[info[#info]].r = r
				mountdb[info[#info]].g = g
				mountdb[info[#info]].b = b
			end,
		},
	}
}